import { afterEvents, checkBlockPattern, Nether, Overworld, sleep, TheEnd, Vector } from "./libraries/utils";
import { BlockPermutation, BlockTypes, system, world } from "@minecraft/server";
import Materials from "./Materials";
const GolemPatterns = [
    /**
     * ###
     *  #
     */
    [
        { offset: { x: 0, y: -1, z: 0 } },
        { offset: { x: 1, y: -1, z: 0 } },
        { offset: { x: -1, y: -1, z: 0 } },
        { offset: { x: 0, y: -2, z: 0 } },
    ],
    [
        { offset: { x: 0, y: -1, z: 0 } },
        { offset: { x: 0, y: -1, z: 1 } },
        { offset: { x: 0, y: -1, z: -1 } },
        { offset: { x: 0, y: -2, z: 0 } },
    ],
    /**
     *  #
     * ###
     */
    [
        { offset: { x: 0, y: 1, z: 0 } },
        { offset: { x: 1, y: 1, z: 0 } },
        { offset: { x: -1, y: 1, z: 0 } },
        { offset: { x: 0, y: 2, z: 0 } },
    ],
    [
        { offset: { x: 0, y: 1, z: 0 } },
        { offset: { x: 0, y: 1, z: 1 } },
        { offset: { x: 0, y: 1, z: -1 } },
        { offset: { x: 0, y: 2, z: 0 } },
    ],
    /**
     * #
     * ##
     * #
     */
    [
        { offset: { x: 1, y: 0, z: 0 } },
        { offset: { x: 2, y: 0, z: 0 } },
        { offset: { x: 1, y: 1, z: 0 } },
        { offset: { x: 1, y: -1, z: 0 } },
    ],
    [
        { offset: { x: 1, y: 0, z: 0 } },
        { offset: { x: 2, y: 0, z: 0 } },
        { offset: { x: 1, y: 0, z: 1 } },
        { offset: { x: 1, y: 0, z: -1 } },
    ],
    /**
     *  #
     * ##
     *  #
     */
    [
        { offset: { x: -1, y: 0, z: 0 } },
        { offset: { x: -2, y: 0, z: 0 } },
        { offset: { x: -1, y: 1, z: 0 } },
        { offset: { x: -1, y: -1, z: 0 } },
    ],
    [
        { offset: { x: -1, y: 0, z: 0 } },
        { offset: { x: -2, y: 0, z: 0 } },
        { offset: { x: -1, y: 0, z: 1 } },
        { offset: { x: -1, y: 0, z: -1 } },
    ],
    /////
    [
        { offset: { x: 0, y: 0, z: 1 } },
        { offset: { x: 0, y: 0, z: 2 } },
        { offset: { x: 0, y: 1, z: 1 } },
        { offset: { x: 0, y: -1, z: 1 } },
    ],
    [
        { offset: { x: 0, y: 0, z: 1 } },
        { offset: { x: 0, y: 0, z: 2 } },
        { offset: { x: 1, y: 0, z: 1 } },
        { offset: { x: -1, y: 0, z: 1 } },
    ],
    /////
    [
        { offset: { x: 0, y: 0, z: -1 } },
        { offset: { x: 0, y: 0, z: -2 } },
        { offset: { x: 0, y: 1, z: -1 } },
        { offset: { x: 0, y: -1, z: -1 } },
    ],
    [
        { offset: { x: 0, y: 0, z: -1 } },
        { offset: { x: 0, y: 0, z: -2 } },
        { offset: { x: 1, y: 0, z: -1 } },
        { offset: { x: -1, y: 0, z: -1 } },
    ],
];
afterEvents.playerPlaceBlock.subscribe(({ player, block, dimension }) => {
    if (block.typeId !== 'minecraft:carved_pumpkin')
        return;
    const origin = Vector.subtract(block.center(), { y: 2.5 });
    for (const pattern of GolemPatterns) {
        const shape = checkBlockPattern(block, pattern);
        if (!shape)
            continue;
        const material = shape[0];
        if (material.isAir || material.isLiquid)
            continue;
        const golemType = Materials[material.typeId];
        if (!golemType)
            continue;
        block.setType('minecraft:air');
        for (const block of shape) {
            block?.setType('minecraft:air');
        }
        dimension.playSound('dig.wood', origin);
        const groundY = shape.reduce((a, b) => a < b.y ? a : b.y, block.location.y);
        const ground = new Vector(origin.x, groundY, origin.z);
        const golem = dimension.spawnEntity(golemType, ground);
        golem.setRotation(Vector.multiply({ ...player.getRotation(), z: 0 }, -1));
        const tameable = golem.getComponent('tameable');
        tameable.tame(player);
        break;
    }
});
const IPattern = [
    { offset: { x: 0, y: -1, z: 0 }, id: 'minecraft:carved_pumpkin' },
    { offset: { x: 0, y: -2, z: 0 }, id: 'minecraft:honeycomb_block' },
];
afterEvents.playerPlaceBlock.subscribe(({ player, block, dimension }) => {
    const match = block.typeId.match(/minecraft:((\D+)_)?candle/);
    if (!match)
        return;
    const origin = Vector.subtract(block.center(), { y: 2.5 });
    const shape = checkBlockPattern(block, IPattern);
    if (!shape)
        return;
    block.setType('minecraft:air');
    for (const block of shape) {
        block?.setType('minecraft:air');
    }
    dimension.playSound('dig.wood', origin);
    const groundY = shape.reduce((a, b) => a < b.y ? a : b.y, block.location.y);
    const ground = new Vector(origin.x, groundY, origin.z);
    const golem = dimension.spawnEntity('t_mg:candle_golem', ground);
    golem.triggerEvent('color_' + (match[2] ?? 'default'));
    golem.setRotation(Vector.multiply({ ...player.getRotation(), z: 0 }, -1));
    const tameable = golem.getComponent('tameable');
    tameable.tame(player);
});
const isOldVersion = !BlockTypes.get('minecraft:light_block_0');
const Light = new Map();
const Dimensions = [Overworld, Nether, TheEnd];
const level = 15;
system.runInterval(() => {
    for (const entity of Dimensions.flatMap(d => d.getEntities({ type: 't_mg:candle_golem' }))) {
        if (!entity?.isValid())
            continue;
        const getHead = () => entity.getHeadLocation();
        SetLight(entity, getHead, level);
        const reduced = Math.max(level - 2, 1);
        // SetLight(entity, () => Vector.add(getHead(), { y: 1 }), reduced);
        SetLight(entity, () => Vector.add(getHead(), { x: 1 }), reduced);
        SetLight(entity, () => Vector.add(getHead(), { x: -1 }), reduced);
        SetLight(entity, () => Vector.add(getHead(), { z: 1 }), reduced);
        SetLight(entity, () => Vector.add(getHead(), { z: -1 }), reduced);
    }
});
function SetLight(player, getLocation, level = 15) {
    const dimension = player.dimension;
    const location = getLocation();
    const block = player.dimension.getBlock(location);
    if (!block?.isValid())
        return;
    const LightBlock = isOldVersion ?
        BlockPermutation.resolve('minecraft:light_block', { block_light_level: level })
        : BlockPermutation.resolve('minecraft:light_block_' + level);
    if ((!block?.isAir && !block?.typeId.match('minecraft:light_block_')) || block?.matches(LightBlock.type.id))
        return;
    const pos = JSON.stringify({
        owner: player.id,
        pos: location,
        dimensionID: dimension.id
    });
    const ID = 'DL:' + pos;
    world.setDynamicProperty(ID, pos);
    block.setPermutation(LightBlock);
    async function OldState() {
        await sleep();
        if (!block?.isValid() || Vector.equals(getLocation(), location) && Light.has(player))
            return OldState();
        world.setDynamicProperty(ID);
        if (block.matches(LightBlock.type.id))
            block.setType('minecraft:air');
    }
    OldState();
}
world.getDynamicPropertyIds().forEach(function NoLight(ID) {
    if (!ID.startsWith('DL:'))
        return;
    try {
        const { pos, dimensionID } = JSON.parse(world.getDynamicProperty(ID));
        const block = world.getDimension(dimensionID).getBlock(pos);
        if (!block)
            return system.run(() => NoLight(ID));
        if (block?.typeId.startsWith('minecraft:light_block'))
            block.setType('minecraft:air');
        world.setDynamicProperty(ID);
    }
    catch { }
});
